import { Component, OnInit, Inject, ElementRef, ViewChild, AfterViewInit } from '@angular/core';
import { WINDOW } from './window.component';
import { SpeechRecognitionService } from './speech-recognition.service';

@Component({
  selector: 'app-char3-d',
  templateUrl: './char3-d.component.html',
  styleUrls: ['./char3-d.component.css']
})
export class Char3DComponent implements OnInit, AfterViewInit {
  textToSay: string = '';
  iframeContent: string = '';
  message: string = '';
  isMacSafari: boolean = false;
  firstEmptyAudioPlayed: boolean = false;

  @ViewChild('frameContentRef') frameContentRef!: ElementRef<HTMLIFrameElement>;
  @ViewChild('statusRef') statusRef!: ElementRef<HTMLParagraphElement>;
  embed: any[] = [16851, 360, 480, "''", 1, 1, 2695840, 0, 1, 1, "'5c6fde06460aa8ab405bf4f13cf93cf0'", 0];
  spn: string = 'xyz';

  constructor(
    @Inject(WINDOW) private window: Window,
    private speechService: SpeechRecognitionService
  ) {}

  ngOnInit(): void {
    this.speechService.initialize(
      () => this.onSpeechStart(),
      (error: any) => this.onSpeechError(error),
      () => this.onSpeechEnd(),
      (event: any) => this.handleSpeechRecognitionResult(event)
    );

    const my_window = this.window as any;
    my_window.vh_aiResponse = this.handleAIResponse.bind(this);
    my_window.vh_talkEnded = this.handleSpeakEnd.bind(this);
  }

  ngAfterViewInit() {}

  handleAIResponse(aiString: string) {
    if (!this.textToSay.trim()) {
      console.error('Input is empty');
      return;
    }

    const aiResponse = aiString;
    const textTo_set = `<br><font color="#FF0000">Q - ${this.textToSay}</font><br><font color="#0000FF">A - ${aiResponse}</font>${this.iframeContent}`;
    this.iframeContent = textTo_set;

    if (this.frameContentRef && this.frameContentRef.nativeElement) {
      const doc = this.frameContentRef.nativeElement.contentDocument;
      doc.open();
      doc.write(`<html><body bgcolor="#FFFFFF">${textTo_set}</body></html>`);
      doc.close();
    }

    this.textToSay = '';
    this.message = ""; 
  }

  handleSpeakEnd (){
    this.startSpeechRecognition();
  }

  handleSpeechRecognitionResult(event: any) {
    let interimTranscript = "";
    let finalTranscript = "";
  
    if (this.speechService.isStopped) {
      return;
    }
  
    for (let i = event.resultIndex; i < event.results.length; ++i) {
      if (event.results[i].isFinal && event.results[i][0].confidence > 0) {
        let transcript = event.results[i][0].transcript.trim();
        finalTranscript += transcript; // Accumulate final transcript
      } else {
        interimTranscript += event.results[i][0].transcript;
      }
    }
    
    if (finalTranscript !== "") {
      this.speechService.stopRecognition(); // Stop recognition after getting final result
      this.textToSay = finalTranscript;
      this.message = finalTranscript;
      this.window['sayAI'](finalTranscript, 1, 1, 3);
      this.onSpeechEnd(); // Ensure status is updated after stopping recognition
    }
  
    this.message = interimTranscript;
  }

  onSpeechStart() {
    if (this.statusRef && this.statusRef.nativeElement) {
      this.statusRef.nativeElement.style.display = "block";
    }
  }

  onSpeechError(error: any) {
    console.error("Speech recognition error:", error);
    this.onSpeechEnd(); // Hide status on error
  }

  onSpeechEnd() {
    if (this.statusRef && this.statusRef.nativeElement) {
      this.statusRef.nativeElement.style.display = "none";
    }
  }

  startSpeechRecognition() {
    if (this.isMacSafari && !this.firstEmptyAudioPlayed) {
      this.window;
    } else {
      this.speechService.startRecognition();
      this.onSpeechStart(); // Ensure status is updated immediately
    }
  }

  stopSpeechRecognition() {
    this.speechService.stopRecognition();
    this.onSpeechEnd(); // Ensure status is updated immediately
  }
}
