import { Injectable } from '@angular/core';

@Injectable({
  providedIn: 'root'
})
export class SpeechRecognitionService {
  private recognition: any;
  private isSpeechStopCalled: boolean = false;

  constructor() {
    if (window.hasOwnProperty('webkitSpeechRecognition') || window.hasOwnProperty('SpeechRecognition')) {
      const SpeechRecognition = window['webkitSpeechRecognition'] || window['SpeechRecognition'];
      this.recognition = new SpeechRecognition();
      this.recognition.continuous = true;
      this.recognition.interimResults = true;
      this.recognition.maxAlternatives = 1;
      this.recognition.lang = 'en-US';
    } else {
      this.handleSpeechRecognitionNotAvailable();
    }
  }

  initialize(onStart: Function, onError: Function, onEnd: Function, onResult: Function) {
    if (this.recognition) {
      this.recognition.onstart = onStart;
      this.recognition.onerror = onError;
      this.recognition.onend = onEnd;
      this.recognition.onresult = onResult;
    }
  }

  startRecognition() {
    if (this.recognition) {
      this.isSpeechStopCalled = false;
      this.recognition.start();
    }
  }

  stopRecognition() {
    if (this.recognition) {
      this.isSpeechStopCalled = true;
      this.recognition.stop();
    }
  }

  handleSpeechRecognitionNotAvailable() {
    console.log("Speech Recognition Not Available");
    this.stopRecognition();
    // Adjust UI or handle logic when Speech Recognition is not available
  }

  get isStopped() {
    return this.isSpeechStopCalled;
  }
}
